#! /usr/bin/env ruby
#
# This script is run after creating the HTML version of the documentation.
# It creates the directory ARGV[0]/images, populates it with the image
# files used by the HTML, and munges the image tags in the HTML to refer
# to the now-local image files.
#

require 'ftools'
require 'find'

dirname = ARGV[0]
alreadySeen = Hash.new()
File.makedirs("#{dirname}/images")
Find.find(dirname) { | f |
    next if File.directory?(f)	# Not really necessary

    File.open(f + ".out", 'w') { | out |
	IO.readlines(f).each { | line |
	    if line =~ /SRC="(file:[^"]+)"/
		path = $1[5..-1] # Skip "file:"
		filename = File.basename(path)

		# If first time, copy image file
		if alreadySeen[path] == nil
		    File.cp(path, "#{dirname}/images")
		    alreadySeen[path] = true
		end

		# Munge image URL
		line.sub!($1, "images/#{filename}")
	    end
	    out.print line	# Copy line from original file to new file
	}
    }
    File.rename(f + ".out", f)	# Move new file over old file
}
