package jimm.datavision;
import jimm.datavision.sql.Column;

/**
 * A column field represents a database column. The value of a column field
 * holds the database column object.
 *
 * @see Column
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class ColumnField extends Field {

protected Column column;

/**
 * Constructs a column field with the specified id in the specified report
 * section whose database column's id is <i>value</i>.
 *
 * @param id the new field's id
 * @param section the report section in which the field resides
 * @param value the string id of a database column
 * @see Column
 */
public ColumnField(Long id, Section section, Object value) {
    super(id, section, value);
    column = Column.find((String)value);
}

public String dragString() {
    return typeString() + ":" + column.getId();
}

/**
 * Returns the database column.
 *
 * @return the database column
 */
public Column getColumn() { return column; }

/**
 * Sets the database column.
 *
 * @param newColumn the new database column
 */
public void setColumn(Column newColumn) {
    if (column != newColumn) {
	column = newColumn;
	setDirty(true);
	setChanged();
	notifyObservers();
    }
}

public String typeString() { return "column"; }

public String formulaString() { return "{" + value + "}"; }

/**
 * Returns the value of this field. For column fields, this is the current
 * value of the database column.
 *
 * @return the value string
 */
public Object getValue() { return section.report.columnValue(column); }

}
