package jimm.datavision;
import java.util.Observable;

/**
 * A point with double coordinates.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class Point extends Observable implements Dirtiable, Writeable {

/**
 * Warning: though public, treat as read-only. When writing, make sure to
 * use setter method or to explicitly set dirty bit when changing x.
 */
public double x;
/**
 * Warning: though public, treat as read-only. When writing, make sure to
 * use setter method or to explicitly set dirty bit when changing y.
 */
public double y;
protected boolean dirty;

/** Constructor. */
public Point() { this(0, 0); }

/**
 * Constructor.
 *
 * @param x a double
 * @param y a double
 */
public Point(double x, double y) {
    this.x = x;
    this.y = y;
}

/**
 * Returns <code>true</code> if this point has been modified.
 *
 * @return <code>true</code> if this point has been modified
 */
public boolean isDirty() { return dirty; }

/**
 * Sets or clears the dirty bit.
 *
 * @param dirty the new value
 */
public void setDirty(boolean dirty) { this.dirty = dirty; }

/**
 * Returns the x coordinate.
 *
 * @return the doubleing-point x coordinate
 */
public double getX() { return x; }

/**
 * Sets the x coordinate.
 *
 * @param newX the new x coordinate
 */
public void setX(double newX) {
    if (x != newX) {
	x = newX;
	setDirty(true);
	setChanged();
	notifyObservers();
    }
}

/**
 * Returns the y coordinate.
 *
 * @return the doubleing-point y coordinate
 */
public double getY() { return y; }

/**
 * Sets the y coordinate.
 *
 * @param newY the new y coordinate
 */
public void setY(double newY) {
    if (y != newY) {
	y = newY;
	setDirty(true);
	setChanged();
	notifyObservers();
    }
}

/**
 * Returns the distance from this point to another.
 *
 * @param p the other point
 * @return the distance between the two points
 */
public double distanceTo(Point p) {
    double dx = p.x - x;
    double dy = p.y - y;
    if (dx == 0) return Math.abs(dy);
    if (dy == 0) return Math.abs(dx);
    return Math.sqrt(dx * dx + dy * dy);
}

/**
 * Returns a string representation of this point.
 *
 * @return a string representing this point
 */
public String toString() {
    return "(" + x + ", " + y + ")";
}

/**
 * Writes this point as an XML tag.
 *
 * @param out a writer that knows about indenting XML tags
 */
public void writeXML(IndentWriter out) {
    out.println("<point x=\"" + x + "\" y=\"" + y + "\" />");
}

}
