package jimm.datavision.gui;
import jimm.datavision.Format;
import jimm.datavision.Field;
import jimm.datavision.Border;
import jimm.datavision.BorderEdge;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.*;

/**
 * A field bounds (position and size) editing dialog box.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class BoundsWin extends EditWin implements FieldWidgetWalker {

class RevertInfo {
FieldWidget fw;
boolean fieldDirty;
boolean boundsDirty;
jimm.datavision.Rectangle bounds;
RevertInfo(FieldWidget fw, boolean fieldDirty, boolean boundsDirty,
	   jimm.datavision.Rectangle bounds)
{
    this.fw = fw;
    this.fieldDirty = fieldDirty;
    this.boundsDirty = boundsDirty;
    this.bounds = new jimm.datavision.Rectangle(bounds);
}
}

protected static final int TEXT_FIELD_COLS = 8;

protected DesignWin designWin;
protected FieldWidget fieldWidget;
protected jimm.datavision.Rectangle fieldBounds;
protected boolean saveRevertInfo;
protected JTextField x_text;
protected JTextField y_text;
protected JTextField w_text;
protected JTextField h_text;

/**
 * Constructor; uses format of first selected field.
 *
 * @param designWin the window to which this dialog belongs
 */
public BoundsWin(DesignWin designWin) {
    this(designWin, designWin.firstSelectedField());
}

/**
 * Constructor.
 *
 * @param designWin the window to which this dialog belongs
 * @param fw a field widget from which we will take the format
 */
public BoundsWin(DesignWin designWin, FieldWidget fw) {
    super(designWin, "Field Position and Size");

    this.designWin = designWin;
    fieldWidget = fw;
    jimm.datavision.Rectangle origBounds = fieldWidget.getField().getBounds();
    fieldBounds = new jimm.datavision.Rectangle(origBounds);

    buildWindow();
    pack();
    setVisible(true);
}

/**
 * Builds the window contents.
 */
protected void buildWindow() {
    JPanel editorPanel = buildBoundsEditor();

    // Ok, Apply, Revert, and Cancel Buttons
    JPanel buttonPanel = closeButtonPanel();

    // Add values and buttons to window
    getContentPane().add(editorPanel, BorderLayout.CENTER);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);
}

protected JPanel buildBoundsEditor() {
    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    x_text = addCoord(panel, "X", fieldBounds.x, BorderLayout.WEST);
    y_text = addCoord(panel, "Y", fieldBounds.y, BorderLayout.NORTH);
    w_text = addCoord(panel, "Width", fieldBounds.width, BorderLayout.EAST);
    h_text = addCoord(panel, "Height", fieldBounds.height, BorderLayout.SOUTH);

    return panel;
}

protected JTextField addCoord(JPanel parent, String label, double value,
			      String compassPoint)
{
    JPanel coordPanel = new JPanel();
    coordPanel.add(new JLabel(label));
    JTextField text = new JTextField("" + value, TEXT_FIELD_COLS);
    coordPanel.add(text);
    parent.add(coordPanel, compassPoint);
    return text;
}

protected void fillCoords(jimm.datavision.Rectangle bounds) {
    x_text.setText("" + bounds.x);
    y_text.setText("" + bounds.y);
    w_text.setText("" + bounds.width);
    h_text.setText("" + bounds.height);
}

protected void doSave(boolean saveRevertInfo) {
    fieldBounds.setBounds(Double.parseDouble(x_text.getText()),
			  Double.parseDouble(y_text.getText()),
			  Double.parseDouble(w_text.getText()),
			  Double.parseDouble(h_text.getText()));

    this.saveRevertInfo = saveRevertInfo;
    if (designWin.countSelectedFields() == 0)
	step(fieldWidget);
    else			// Call step() for all selected fields
	designWin.withSelectedWidgetsDo(this);
}

/**
 * Saves a single field widget.
 *
 * @param fw a field widget
 */
public void step(FieldWidget fw) {
    // Create revert info
    if (saveRevertInfo && !revertInfoList.containsKey(fw)) {
	Field f = fw.getField();
	jimm.datavision.Rectangle origBounds = f.getBounds();
	RevertInfo ri = new RevertInfo(fw, f.isDirty(), origBounds.isDirty(),
				       origBounds);
	revertInfoList.put(fw, ri);
    }

    // Save new value
    fw.applyNewBounds(fieldBounds);
}

protected void doRevert(Object revertInfo) {
    RevertInfo ri = (RevertInfo)revertInfo;

    ri.fw.applyNewBounds(ri.bounds);

    Field f = ri.fw.getField();
    f.getBounds().setBounds(ri.bounds);
    f.getBounds().setDirty(ri.boundsDirty);
    f.setDirty(ri.fieldDirty);

    fillCoords(ri.bounds);	// Revert GUI
}

}
