package jimm.datavision.gui;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;

/**
 * This is the abstract superclass of windows used for editing
 * paragraphs of code such as formulas and where clauses. The text
 * field accepts dragged report fields.
 *
 * @see DropListenerTextArea
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public abstract class CodeEditorWin extends JDialog implements ActionListener {

protected static final Dimension EDIT_SIZE = new Dimension(400, 225);

protected JTextArea codeField;

/**
 * Constructor.
 *
 * @param frame the window to which this dialog belongs
 * @param initialText the initial text to edit
 * @param title the window title
 */
public CodeEditorWin(JFrame frame, String initialText, String title) {
    super(frame, title);
    buildWindow(initialText);
    pack();
    setVisible(true);
}

/**
 * Builds the window contents.
 *
 * @param whichTab the index of the tab to display when opened
 */
protected void buildWindow(String initialText) {
    // Editor panel containing where clause edit field
    JPanel editorPanel = new JPanel();
    codeField =
	new DropListenerTextArea(initialText == null ? "" : initialText);
    codeField.setBorder(BorderFactory.createLoweredBevelBorder());
    editorPanel.add(codeField);

    // Set preferred size for editor
    codeField.setPreferredSize(EDIT_SIZE);

    // Ok and Cancel Buttons
    JPanel buttonPanel = buildButtonPanel();

    // Add edit panel and buttons to window
    getContentPane().add(editorPanel, BorderLayout.CENTER);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    new FocusSetter(codeField);
}

/**
 * Builds and returns a panel containing the OK and Cancel
 *
 * @return a panel
 */
protected JPanel buildButtonPanel() {
    JPanel buttonPanel = new JPanel();
    JButton button;

    buttonPanel.add(button = new JButton("Ok"));
    button.addActionListener(this);
    button.setDefaultCapable(true);

    buttonPanel.add(button = new JButton("Cancel"));
    button.addActionListener(this);

    return buttonPanel;
}

/**
 * Handles the OK and Cancel buttons.
 *
 * @param e action event
 */
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if ("Ok".equals(cmd)) {
	save(codeField.getText());
	dispose();
    }
    else if ("Cancel".equals(cmd)) {
	dispose();
    }
}

/**
 * Implement this to do whatevery you gotta do with the text.
 *
 * @param text the text in the edit box
 */
public abstract void save(String text);

}
