package jimm.datavision.gui;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.*;

/**
 * A modal dialog used to ask the user for a database password.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class DbPasswordDialog extends JDialog implements ActionListener {

protected String username;
protected String password;
protected JTextField usernameField;
protected JPasswordField passwordField;

/**
 * Constructor.
 *
 * @param parent frame with which this dialog should be associated
 * @param dbName database name
 * @param userName database user name
 */
public DbPasswordDialog(Frame parent, String dbName, String userName) {
    super(parent, "Database Password", true); // Modal
    username = userName == null ? "" : userName;
    buildWindow(dbName);
    pack();
    setVisible(true);
}

/**
 * Returns username (or <code>null</code> if user hit Cancel).
 *
 * @return username (or <code>null</code> if user cancelled)
 */
public String getUserName() { return username; }

/**
 * Returns password (or <code>null</code> if user hit Cancel).
 *
 * @return password (or <code>null</code> if user cancelled)
 */
public String getPassword() { return password; }

protected void buildWindow(String dbName) {
    getContentPane().setLayout(new BorderLayout());

    JPanel labelPanel = new JPanel();
    labelPanel.setLayout(new GridLayout(0, 1));
    labelPanel.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 2));

    // Labels on left side
    JLabel label;
    labelPanel.add(label = new JLabel("Database:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);
    labelPanel.add(label = new JLabel("User Name:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);
    labelPanel.add(label = new JLabel("Password:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);

    JPanel fieldPanel = new JPanel();
    fieldPanel.setLayout(new GridLayout(0, 1));
    fieldPanel.setBorder(BorderFactory.createEmptyBorder(8, 2, 8, 8));

    fieldPanel.add(new JLabel(dbName));
    fieldPanel.add(usernameField = new JTextField(username));
    fieldPanel.add(passwordField = new JPasswordField());

    JPanel editPanel = new JPanel();
    editPanel.setLayout(new BorderLayout());
    editPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
    editPanel.add(labelPanel, BorderLayout.CENTER);
    editPanel.add(fieldPanel, BorderLayout.EAST);

    JPanel buttonPanel = new JPanel();
    JButton button;

    buttonPanel.add(button = new JButton("Ok"));
    button.addActionListener(this);
    button.setDefaultCapable(true);
    getRootPane().setDefaultButton(button);

    buttonPanel.add(button = new JButton("Cancel"));
    button.addActionListener(this);

    getContentPane().add(editPanel, BorderLayout.CENTER);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	    dispose();
	}
	});

    new FocusSetter(passwordField);
}

/**
 * Handles the buttons.
 *
 * @param e action event
 */
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if ("Ok".equals(cmd)) {
	username = usernameField.getText();
	password = new String(passwordField.getPassword());
	dispose();
    }
    else if ("Cancel".equals(cmd)) {
	dispose();
    }
}

}
