package jimm.datavision.gui;
import jimm.datavision.Report;
import jimm.datavision.ErrorHandler;
import jimm.datavision.sql.Database;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.ArrayList;
import javax.swing.*;

/**
 * A report description (name, title, author, etc.) editing dialog box.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class DescripWin extends EditWin {

class RevertInfo {
boolean reportDirty;
String name;
String title;
String author;
String description;
RevertInfo(Report r)
{
    reportDirty = r.isDirty();
    name = report.getName();
    title = report.getTitle();
    author = report.getAuthor();
    description = report.getDescription();
}
}

protected static final int TEXT_FIELD_COLS = 32;
protected static final int TEXT_AREA_ROWS = 8;
protected static final int TEXT_AREA_COLS = 32;

protected DesignWin designWin;
protected Report report;
protected JTextField nameField;
protected JTextField titleField;
protected JTextField authorField;
protected JTextArea descriptionField;

/**
 * Constructor.
 *
 * @param designWin the window to which this dialog belongs
 * @param database report database connection
 */
public DescripWin(DesignWin designWin, Report report) {
    super(designWin, "Report Description");

    this.designWin = designWin;
    this.report = report;

    buildWindow();
    pack();
    setVisible(true);
}

/**
 * Builds the window contents.
 */
protected void buildWindow() {
    // All edit fields
    JPanel editorPanel = buildEditor();

    // Ok, Apply, Revert, and Cancel Buttons
    JPanel buttonPanel = closeButtonPanel();

    // Add values and buttons to window
    getContentPane().add(editorPanel, BorderLayout.CENTER);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);
}

protected JPanel buildEditor() {
    GridBagLayout bag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.insets = new Insets(6, 6, 6, 6);

    JPanel panel = new JPanel();
    panel.setLayout(bag);
    panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

    nameField = buildField(0, "Report Name:", report.getName(),
			   panel, bag, c);
    titleField = buildField(1, "Report Title:", report.getTitle(),
			    panel, bag, c);
    authorField = buildField(2, "Author's Name:", report.getAuthor(),
			     panel, bag, c);
    descriptionField = buildArea(3, "Description:", report.getDescription(),
				 panel, bag, c);

    return panel;
}

protected JTextField buildField(int row, String labelText, String val,
				JPanel panel, GridBagLayout bag,
				GridBagConstraints c)
{
    JLabel label = new JLabel(labelText);
    c.gridx = 0;
    c.gridy = row;
    c.anchor = GridBagConstraints.NORTHEAST;
    bag.setConstraints(label, c);
    panel.add(label);

    JTextField field = new JTextField(val == null ? "" : val, TEXT_FIELD_COLS);
    c.gridx = 1;
    c.anchor = GridBagConstraints.NORTHWEST;
    bag.setConstraints(field, c);
    panel.add(field);
    return field;
}

protected JTextArea buildArea(int row, String labelText, String val,
			      JPanel panel, GridBagLayout bag,
			      GridBagConstraints c)
{
    JLabel label = new JLabel(labelText);
//      label.setHorizontalAlignment(SwingConstants.RIGHT);
    c.gridx = 0;
    c.gridy = row;
    c.anchor = GridBagConstraints.NORTHEAST;
    bag.setConstraints(label, c);
    panel.add(label);

    JTextArea field = new JTextArea(val == null ? "" : val, TEXT_AREA_ROWS,
				    TEXT_AREA_COLS);
    field.setBorder(BorderFactory.createLoweredBevelBorder());
    c.gridx = 1;
//      c.gridheight = 4;
    c.anchor = GridBagConstraints.NORTHWEST;
    c.fill = GridBagConstraints.BOTH;
    bag.setConstraints(field, c);
    panel.add(field);
    return field;
}

protected void fillEditFields(String name, String title, String author,
			      String description)
{
    nameField.setText(name);
    titleField.setText(title);
    authorField.setText(author);
    descriptionField.setText(description);
}

protected void doSave(boolean saveRevertInfo) {
    if (saveRevertInfo && !revertInfoList.containsKey(this))
	revertInfoList.put(this, new RevertInfo(report));

    report.setName(nameField.getText());
    report.setTitle(titleField.getText());
    report.setAuthor(authorField.getText());
    report.setDescription(descriptionField.getText());
}

protected void doRevert(Object revertInfo) {
    RevertInfo ri = (RevertInfo)revertInfo;

    report.setName(ri.name);
    report.setTitle(ri.title);
    report.setAuthor(ri.author);
    report.setDescription(ri.description);
    report.setDirty(ri.reportDirty);

    // Revert GUI
    fillEditFields(ri.name, ri.title, ri.author, ri.description);
}

}
