package jimm.datavision.gui;
import jimm.datavision.*;
import jimm.datavision.sql.Column;
import jimm.datavision.gui.parameter.ParamEditWin;
import javax.swing.JFrame;

/**
 * The classes in this file are leaf nodes for the field picker tree.
 *
 * @see FieldPickerTree
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */

// ================================================================
abstract class FPLeafInfo {
protected Draggable leaf;
protected JFrame editorParent;
FPLeafInfo(Draggable obj, JFrame edParent) {
    leaf = obj;
    editorParent = edParent;
}
Object getLeaf() { return leaf; }
public String toString() { return leaf.toString(); }
String dragString() { return leaf.dragString(); }
abstract boolean isDeletable();
void openEditor() {}		// Only interesting for formulas and parameters
}

// ================================================================
class ColumnInfo extends FPLeafInfo {
ColumnInfo(Column c, JFrame edParent) { super(c, edParent); }
public String toString() { return ((Column)leaf).getName(); }
boolean isDeletable() { return false; }
}

// ================================================================
class FormulaInfo extends FPLeafInfo {
protected Report report;
FormulaInfo(Report r, Formula f, JFrame edParent) {
    super(f, edParent);
    report = r;
}
public String toString() { return ((Formula)leaf).getName(); }
boolean isDeletable() { return !report.containsReferenceTo((Formula)leaf); }
void openEditor() { new FormulaWin(editorParent, (Formula)leaf); }
}

// ================================================================
class ParameterInfo extends FPLeafInfo {
protected Report report;
ParameterInfo(Report r, Parameter p, JFrame edParent) {
    super(p, edParent);
    report = r;
}
public String toString() { return ((Parameter)leaf).getName(); }
boolean isDeletable() { return !report.containsReferenceTo((Parameter)leaf); }
void openEditor() { new ParamEditWin(editorParent, (Parameter)leaf); }
}

// ================================================================
class SpecialInfo extends FPLeafInfo {
protected String specialString;
protected String dragString;
SpecialInfo(String ss, String ds, JFrame edParent) {
    super(null, edParent);
    specialString = ss;
    dragString = ds;
}
public String toString() { return specialString; }
public String dragString() { return dragString; }
boolean isDeletable() { return false; }
}

//  class SubtotalInfo extends FPLeafInfo {
//  SubtotalInfo(SubtotalField f, JFrame edParent) { super(f, edParent); }
//  public String toString() { return f.designLabel(); }
//  boolean isDeletable() { return false; }
//  }
