package jimm.datavision.gui;
import jimm.datavision.Formula;
import jimm.datavision.FormulaField;
import jimm.datavision.ErrorHandler;
import java.util.Observable;
import java.util.Observer;

/**
 * This dialog is for editing formula code.
 *
 * @see FormulaWidget
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class FormulaWin extends CodeEditorWin implements Observer {

protected Formula formula;

/**
 * Constructor.
 *
 * @param frame the window to which this dialog belongs
 * @param formula the formula whose text needs editing
 */
public FormulaWin(javax.swing.JFrame frame, Formula formula) {
    super(frame, formula.getEditableEvalString(),
	  "Formula " + formula.getName());
    this.formula = formula;
    formula.addObserver(this);
}

public void finalize() {
    formula.deleteObserver(this);
}

public void update(Observable o, Object arg) {
    setTitle("Formula " + formula.getName());
    codeField.setText(formula.getEditableEvalString());
}

public void save(String text) {
    formula.deleteObserver(this);
    try {
	formula.setEditableEvalString(text);
    }
    catch (IllegalArgumentException iae) {
	ErrorHandler.error(iae.toString() + "\n" + "The formula is unchanged.",
			   "Formula Contents Error");
    }
}

}
