package jimm.datavision.gui;
import jimm.datavision.ErrorHandler;
import java.awt.Dimension;
import java.io.IOException;
import java.net.URL;
import javax.swing.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;

/**
 * A help window. Opens on docs/DataVision/DataVision.html.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class HelpWin extends JFrame implements HyperlinkListener {

protected static final int START_WIDTH = 400;
protected static final int START_HEIGHT = 500;
protected static final String DOCS_DIR = "docs";
protected static final String HTML_DIR = "DataVision";
protected static final String START_CONTENT_FILE = "DataVision.html";

protected static Object instanceSyncLock = new Object();
protected static HelpWin helpWin;

protected JEditorPane contentEditorPane;

public static HelpWin instance() {
    if (helpWin == null) {
	synchronized(instanceSyncLock) {
	    if (helpWin == null)
		helpWin = new HelpWin();
	}
    }
    return helpWin;
}

protected HelpWin() {
    super("DataVision Help");
    buildWindow();
    pack();
    setVisible(true);
}

protected void buildWindow() {
    JComponent contentPane = buildContentPane();
    getContentPane().add(contentPane);
}

protected JComponent buildContentPane() {
    contentEditorPane = new JEditorPane();
    contentEditorPane.setEditable(false);

    // Create a URL object for the file docs/DataVision/DataVision.html.
    String s = null;
    URL helpURL = null;
    try {
	String sep = System.getProperty("file.separator");
	s = "file:" + System.getProperty("user.dir") + sep + DOCS_DIR + sep
	    + HTML_DIR + sep + START_CONTENT_FILE;
	helpURL = new URL(s);
    } catch (Exception e) {
	ErrorHandler.error("Couldn't create help URL: " + s);
    }

    // Load the URL.
    if (helpURL != null) {
	try {
	    contentEditorPane.setPage(helpURL);
	} catch (IOException e) {
	    ErrorHandler.error("Attempted to read a bad help file URL: "
			       + helpURL);
	}
    }

    // Scroll pane.
    JScrollPane scrollPane = new JScrollPane(contentEditorPane);
    scrollPane.setVerticalScrollBarPolicy(
	JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane.setPreferredSize(new Dimension(START_WIDTH, START_HEIGHT));

    // Start listening to hyperlink events.
    contentEditorPane.addHyperlinkListener(this);

    return scrollPane;
}

public void hyperlinkUpdate(HyperlinkEvent e) {
    if (e.getEventType() != HyperlinkEvent.EventType.ACTIVATED)
	return;

    if (e instanceof HTMLFrameHyperlinkEvent) {
	HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
	HTMLDocument doc = (HTMLDocument)contentEditorPane.getDocument();
	doc.processHTMLFrameHyperlinkEvent(evt);
    }
    else {
	try {
	    contentEditorPane.setPage(e.getURL());
	}
	catch (Throwable t) {
	    ErrorHandler.error(t);
	}
    }
}

}
