package jimm.datavision.gui.parameter;
import jimm.datavision.Parameter;
import jimm.datavision.gui.DesignWin;
import jimm.datavision.gui.EditWin;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.CardLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.Observer;
import java.util.Observable;
import java.util.Iterator;
import java.util.Enumeration;
import javax.swing.*;
import com.toedter.calendar.JCalendar;

/**
 * A parameter editing dialog box.
 * <p>
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class ParamEditWin extends EditWin implements Observer {

protected static final int HORIZ_GAP = 20;
protected static final int VERT_GAP = 20;
protected static final int TEXT_FIELD_COLS = 24;
protected static final String CARD_NAME_SINGLE = "single";
protected static final String CARD_NAME_SINGLE_BOOL = "single-bool";
protected static final String CARD_NAME_RANGE = "range";
protected static final String CARD_NAME_LIST = "list";
protected static final String CARD_NAME_DATE = "date";

// ================================================================
class RevertInfo {

Parameter parameter;

RevertInfo(Parameter param)
{
    parameter = (Parameter)param.clone();
}
}
// ================================================================

protected Parameter param;
protected JTextField nameField;
protected JTextField questionField;
protected JRadioButton boolRButton;
protected JRadioButton stringRButton;
protected JRadioButton numericRButton;
protected JRadioButton dateRButton;
protected JRadioButton singleRButton;
protected JRadioButton rangeRButton;
protected JRadioButton listSingleRButton;
protected JRadioButton listMultipleRButton;
protected JPanel cardPanel;
protected JTextField singleField;
protected JCalendar singleDate;
protected JTextField rangeFromField;
protected JTextField rangeToField;
protected JCalendar rangeFromDate;
protected JCalendar rangeToDate;
protected JRadioButton boolYesRButton;
protected JRadioButton boolNoRButton;
protected JList list;
protected JTextField listAddField;

/**
 * Constructor.
 *
 * @param parent the window to which this dialog belongs
 * @param p a parameter
 */
public ParamEditWin(JFrame parent, Parameter p) {
    super(parent, "Parameter");

    param = p;
    param.addObserver(this);

    buildWindow();
    pack();
    setVisible(true);
}

public void update(Observable o, Object arg) {
    fillEditWidgets();
}

/**
 * Builds the window contents.
 *
 * @param whichTab the index of the tab to display when opened
 */
protected void buildWindow() {
    // Add edit panes and buttons to window
    getContentPane().add(buildPromptPanel(), BorderLayout.NORTH);
    getContentPane().add(buildCenterPanel(), BorderLayout.CENTER);

    // Add Ok, Apply, Revert, and Cancel Buttons
    getContentPane().add(closeButtonPanel(), BorderLayout.SOUTH);

    fillEditWidgets();
}

protected JPanel buildPromptPanel() {

    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    // Labels
    JPanel labelPanel = new JPanel();
    labelPanel.setLayout(new GridLayout(0, 1));
    JLabel label;
    labelPanel.add(label = new JLabel("Name:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);
    labelPanel.add(label = new JLabel("Question:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);

    // Edit values
    JPanel fieldPanel = new JPanel();
    fieldPanel.setLayout(new GridLayout(0, 1));
    fieldPanel.add(nameField = new JTextField(TEXT_FIELD_COLS));
    fieldPanel.add(questionField = new JTextField(TEXT_FIELD_COLS));

    panel.add(labelPanel, BorderLayout.WEST);
    panel.add(fieldPanel, BorderLayout.CENTER);

    JPanel nonStretchyPanel = new JPanel();
    nonStretchyPanel.add(panel);
    return nonStretchyPanel;
}

protected JPanel buildCenterPanel() {
    JPanel panel = new JPanel();
    Box box = Box.createHorizontalBox();

    box.add(buildRadioButtonsPanel());
    box.add(buildCardPanel());

    panel.add(box);
    return panel;
}

protected JPanel buildRadioButtonsPanel() {
    JPanel panel = new JPanel();

    // Type radio buttons
    Box box = Box.createVerticalBox();
    ButtonGroup bg = new ButtonGroup();
    box.add(new JLabel("Type"));
    boolRButton = addRadioButton("Yes/No", box, bg);
    stringRButton = addRadioButton("Text", box, bg);
    numericRButton = addRadioButton("Number", box, bg);
    dateRButton = addRadioButton("Date", box, bg);
    panel.add(box);

    // Arity radio buttons
    box = Box.createVerticalBox();
    box.add(new JLabel("Arity"));
    bg = new ButtonGroup();
    singleRButton = addRadioButton("Single", box, bg);
    rangeRButton = addRadioButton("Range", box, bg);
    listSingleRButton = addRadioButton("List (Single Choice)", box, bg);
    listMultipleRButton = addRadioButton("List (Multiple)", box, bg);
    panel.add(box);

    return panel;
}

protected JPanel buildCardPanel() {
    cardPanel = new JPanel();
    cardPanel.setLayout(new CardLayout(HORIZ_GAP, VERT_GAP));

    cardPanel.add(boolCard(), CARD_NAME_SINGLE_BOOL);
    cardPanel.add(singleCard(), CARD_NAME_SINGLE);
    cardPanel.add(rangeCard(), CARD_NAME_RANGE);
    cardPanel.add(listCard(), CARD_NAME_LIST);
    cardPanel.add(dateCard(), CARD_NAME_DATE);

    return cardPanel;
}

protected JPanel dateCard() {
    JPanel panel = new JPanel();
    panel.add(new JLabel("The current date will be the default value."));
    return panel;
}

protected JPanel boolCard() {
    JPanel panel = new JPanel();
    panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

    Box box = Box.createVerticalBox();
    ButtonGroup bg = new ButtonGroup();

    boolYesRButton = new JRadioButton("Yes");
    box.add(boolYesRButton);
    bg.add(boolYesRButton);

    boolNoRButton = new JRadioButton("No");
    box.add(boolNoRButton);
    bg.add(boolNoRButton);

    panel.add(box);
    return panel;
}

protected JPanel singleCard() {
    JPanel panel = new JPanel();
    panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

    panel.add(new JLabel("Default Value:"));
    panel.add(singleField = new JTextField(TEXT_FIELD_COLS));

    return panel;
}

protected JPanel rangeCard() {
    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    // Labels
    JPanel labelPanel = new JPanel();
    labelPanel.setLayout(new GridLayout(0, 1));
    JLabel label;
    labelPanel.add(label = new JLabel("From:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);
    labelPanel.add(label = new JLabel("To:"));
    label.setHorizontalAlignment(SwingConstants.RIGHT);

    // Edit values
    JPanel fieldPanel = new JPanel();
    fieldPanel.setLayout(new GridLayout(0, 1));

    // From and to fields
    fieldPanel.add(rangeFromField = new JTextField(TEXT_FIELD_COLS));
    fieldPanel.add(rangeToField = new JTextField(TEXT_FIELD_COLS));

    panel.add(labelPanel, BorderLayout.CENTER);
    panel.add(fieldPanel, BorderLayout.EAST);

    return panel;
}

protected JPanel listCard() {
    JPanel panel = new JPanel();
    Box box = Box.createVerticalBox();
    panel.add(box);

    list = new JList(new DefaultListModel());
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    box.add(new JScrollPane(list));

    JPanel addPanel = new JPanel();
    JButton button = new JButton("Add to List");
    button.addActionListener(this);
    addPanel.add(button);
    addPanel.add(listAddField = new JTextField(TEXT_FIELD_COLS));
    box.add(addPanel);

    button = new JButton("Remove Selected from List");
    button.addActionListener(this);
    box.add(button);

    return panel;
}

/**
 * Creates and adds a radio buton
 *
 * @param name the button name (label)
 * @param parent a container
 * @param group the button group
 * @return the new radio button
 */
protected JRadioButton addRadioButton(String name, Container parent,
				      ButtonGroup group)
{
    JRadioButton button = new JRadioButton(name);
    button.addActionListener(this);
    parent.add(button);
    group.add(button);
    return button;
}

protected void fillEditWidgets() {
    nameField.setText(param.getName());
    questionField.setText(param.getQuestion());

    // Select type radio button.
    int type = param.getType();
    switch (type) {
    case Parameter.TYPE_BOOLEAN:
	boolRButton.setSelected(true);
	break;
    case Parameter.TYPE_STRING:
	stringRButton.setSelected(true);
	break;
    case Parameter.TYPE_NUMERIC:
	numericRButton.setSelected(true);
	break;
    case Parameter.TYPE_DATE:
	dateRButton.setSelected(true);
	break;
    }

    // Enable only legal arity radio buttons.
    singleRButton.setEnabled(param.isLegal(type, Parameter.ARITY_ONE));
    rangeRButton.setEnabled(param.isLegal(type, Parameter.ARITY_RANGE));
    listSingleRButton
	.setEnabled(param.isLegal(type, Parameter.ARITY_LIST_SINGLE));
    listMultipleRButton
	.setEnabled(param.isLegal(type, Parameter.ARITY_LIST_MULTIPLE));

    // Based on arity, select the appropriate radio button.
    switch (param.getArity()) {
    case Parameter.ARITY_ONE:
	singleRButton.setSelected(true);
	break;
    case Parameter.ARITY_RANGE:
	rangeRButton.setSelected(true);
	break;
    case Parameter.ARITY_LIST_SINGLE:
	listSingleRButton.setSelected(true);
	break;
    case Parameter.ARITY_LIST_MULTIPLE:
	listMultipleRButton.setSelected(true);
	break;
    }

    selectAndFillCard();	// Select card and fill card contents
}

/**
 * Based on type and arity, selects proper card and fills card contents.
 * Since we could be switching from one parameter type to another, we
 * have to do some checking along the way to see if the parameter's default
 * value is of the correct type.
 */
protected void selectAndFillCard() {
    CardLayout cardLayout = (CardLayout)cardPanel.getLayout();
    Object objVal;
    int type = param.getType();
    switch (param.getArity()) {
    case Parameter.ARITY_ONE:
	switch (type) {
	case Parameter.TYPE_BOOLEAN:
	    cardLayout.show(cardPanel, CARD_NAME_SINGLE_BOOL);
	    objVal = param.getDefaultValue(0);
	    Boolean val = null;
	    val = (objVal instanceof Boolean) ? (Boolean)objVal
		: (Boolean)param.getDefaultForType(type);
	    if (val != null && val.booleanValue())
		boolYesRButton.setSelected(true);
	    else
		boolNoRButton.setSelected(true);
	    break;
	case Parameter.TYPE_DATE:
	    cardLayout.show(cardPanel, CARD_NAME_DATE);
	    break;
	default:
	    cardLayout.show(cardPanel, CARD_NAME_SINGLE);
	    setField(singleField, 0);
	    break;
	}
	break;
    case Parameter.ARITY_RANGE:
	if (param.getType() == Parameter.TYPE_DATE)
	    cardLayout.show(cardPanel, CARD_NAME_DATE);
	else
	    cardLayout.show(cardPanel, CARD_NAME_RANGE);
	setField(rangeFromField, 0);
	setField(rangeToField, 1);
	break;
    case Parameter.ARITY_LIST_SINGLE:
    case Parameter.ARITY_LIST_MULTIPLE:
	cardLayout.show(cardPanel, CARD_NAME_LIST);

	// Erase list and re-fill it
	DefaultListModel model = (DefaultListModel)list.getModel();
	model.clear();
	for (Iterator iter = param.defaultValues(); iter.hasNext(); )
	    model.addElement(iter.next());
	break;
    }
}

/**
 * Fills the specified text field with the nth parameter default value.
 * If the value is <code>null</code>, "fill" the text field with the
 * empty string.
 *
 * @param f the text field
 * @param which n
 */
protected void setField(JTextField f, int which) {
    Object obj = param.getDefaultValue(which); // Default value
    f.setText(obj == null ? "" : obj.toString());
}

/**
 * Handles radio buttons.
 */
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    // Changes of type.
    if (cmd.equals("Yes/No")) {
	if (!param.isLegal(Parameter.TYPE_BOOLEAN, param.getArity()))
	    param.setArity(Parameter.ARITY_ONE);
	param.setType(Parameter.TYPE_BOOLEAN);
    }
    else if (cmd.equals("Text")) {
	if (!param.isLegal(Parameter.TYPE_STRING, param.getArity()))
	    param.setArity(Parameter.ARITY_ONE);
	param.setType(Parameter.TYPE_STRING);
    }
    else if (cmd.equals("Number")) {
	if (!param.isLegal(Parameter.TYPE_NUMERIC, param.getArity()))
	    param.setArity(Parameter.ARITY_ONE);
	param.setType(Parameter.TYPE_NUMERIC);
    }
     else if (cmd.equals("Date")) {
	if (!param.isLegal(Parameter.TYPE_DATE, param.getArity()))
	    param.setArity(Parameter.ARITY_ONE);
	param.setType(Parameter.TYPE_DATE);
     }

    // Changes of arity. The user can't pick an illegal combination because
    // we make sure of that in fillEditWidgets().
    else if (cmd.equals("Single"))
	param.setArity(Parameter.ARITY_ONE);
    else if (cmd.equals("Range")) {
	param.setArity(Parameter.ARITY_RANGE);
    }
    else if (cmd.equals("List (Single Choice)"))
	param.setArity(Parameter.ARITY_LIST_SINGLE);
    else if (cmd.equals("List (Multiple)"))
	param.setArity(Parameter.ARITY_LIST_MULTIPLE);
    else if (cmd.equals("Add to List")) {
	((DefaultListModel)list.getModel()).addElement(listAddField.getText());
	listAddField.setText("");
    }

    else if (cmd.equals("Remove Selected from List")) {
	((DefaultListModel)list.getModel())
	    .removeElement(list.getSelectedValue());
    }
    else {
	super.actionPerformed(e);
    }
}

public void dispose() {
    param.deleteObserver(this);
    super.dispose();
}

protected void doSave(boolean saveRevertInfo) {
    if (saveRevertInfo && !revertInfoList.containsKey(param)) {
	RevertInfo ri = new RevertInfo(param);
	revertInfoList.put(param, ri);
    }

    // We've been modifying the parameter's original type and airty
    // directly. Update the remaining fields.
    param.setName(nameField.getText());
    param.setQuestion(questionField.getText());

    param.removeDefaultValues();
    switch (param.getArity()) {
    case Parameter.ARITY_ONE:
	switch (param.getType()) {
	case Parameter.TYPE_BOOLEAN:
	    param.addDefaultValue(new Boolean(boolYesRButton.isSelected()));
	    break;
	case Parameter.TYPE_DATE:
	    // Nothing to do; the date inquisitors will fill in the current
	    // date.
	    break;
	default:
	    param.addDefaultValue(singleField.getText());
	    break;
	}
	break;
    case Parameter.ARITY_RANGE:
	if (param.getType() != Parameter.TYPE_DATE) {
	    param.addDefaultValue(rangeFromField.getText());
	    param.addDefaultValue(rangeToField.getText());
	}
	break;
    case Parameter.ARITY_LIST_SINGLE:
    case Parameter.ARITY_LIST_MULTIPLE:
	DefaultListModel model = (DefaultListModel)list.getModel();
	for (Enumeration enum = model.elements(); enum.hasMoreElements(); )
	    param.addDefaultValue(enum.nextElement());
	break;
    }
}

protected void doRevert(Object revertInfo) {
    RevertInfo ri = (RevertInfo)revertInfo;
    Parameter orig = ri.parameter;

    param.setName(orig.getName());
    param.setQuestion(orig.getQuestion());
    param.setType(orig.getType());
    param.setArity(orig.getArity());

    switch (param.getArity()) {
    case Parameter.ARITY_ONE:
	param.addDefaultValue(singleField.getText());
	break;
    case Parameter.ARITY_RANGE:
	param.addDefaultValue(rangeFromField.getText());
	param.addDefaultValue(rangeToField.getText());
	break;
    case Parameter.ARITY_LIST_SINGLE:
	for (Iterator iter = orig.defaultValues(); iter.hasNext(); )
	    param.addDefaultValue(iter.next());
	break;
    case Parameter.ARITY_LIST_MULTIPLE:
	for (Iterator iter = orig.defaultValues(); iter.hasNext(); )
	    param.addDefaultValue(iter.next());
	break;
    }

    param.setDirty(orig.isDirty());
}

}
