package jimm.datavision.layout;
import java.io.PrintWriter;
import jimm.datavision.*;

/**
 * An HTML layout engine.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class HTMLLE extends LayoutEngine {

protected int rowX;

/**
 * Constructor.
 *
 * @param out a print writer
 */
public HTMLLE(PrintWriter out) {
    super(out);
}

protected void doStart() {
    out.println("<html>");
    String title = report.getTitle();
    if (title != null && title.length() > 0)
	out.println("<title>" + report.getTitle() + "</title>");
    out.println("<body bgcolor=\"white\">");
    out.println("<table width=\"" + pageWidth() + "\">");
}

protected void doEnd() {
    out.println("</body>");
    out.println("</table>");
    out.println("</html>");
    out.flush();
}

protected void doOutputSection(Section s) {
    out.println("<tr height=\"#{sect.height}\">");
    rowX = 0;

    super.doOutputSection(s);

    // Pad remaining page width
    if (rowX < pageWidth())
	out.println("<td width=\"" + (pageWidth() - rowX) + "\">&nbsp;</td>");
    out.println("</tr>");
}

protected void doOutputField(Field field) {
    String str = field.toString();
    if (str == null || str.length() == 0)
	str = "&nbsp;";

    Format format = field.getFormat();
    if (format == null)
	format = Format.DEFAULT;
    Rectangle bounds = field.getBounds();

    if (bounds.x > rowX)
	out.println("<td width=\"" + (bounds.x - rowX) + "\">&nbsp;</td>");
    rowX = (int)(bounds.x + bounds.width);

    String align = null;
    switch (format.getAlign()) {
    case Format.ALIGN_LEFT: align = "left"; break;
    case Format.ALIGN_CENTER: align = "center"; break;
    case Format.ALIGN_RIGHT: align = "right"; break;
    }

    out.print("<td align=\"" + align + "\" width=\"" + (int)bounds.width
	      + "\" height=\"" + (int)bounds.height + "\"");
    if (!format.isWrap())
	out.print(" nowrap");
    out.print(">");

    out.print("<font size=\"#{format.size}\">");
    if (format.isBold()) out.print("<b>");
    if (format.isItalic()) out.print("<i>");
    if (format.isUnderline()) out.print("<u>");

    out.print(Utils.escapeHTML(str));

    if (format.isUnderline()) out.print("</u>");
    if (format.isItalic()) out.print("</i>");
    if (format.isBold()) out.print("</b>");
    out.print("</font>");

    out.println("</td>");
}

protected void doOutputLine(Line line) {}

}

