package jimm.datavision.layout;
import jimm.datavision.*;
import java.awt.Font;
import java.awt.Color;
import java.awt.Dimension;
import javax.swing.JTextPane;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;
import javax.swing.text.StyleConstants;

/**
 * A swing field is the visual representation of a report field.
 *
 * @see SwingLE
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class SwingField extends JTextPane /* implements LineDrawer */ {

protected Field field;

/**
 * Constructor.
 *
 * @param f report field
 */
public SwingField(Field f) {
    this(f, f.toString());
}

/**
 * Constructor.
 *
 * @param f report field
 * @param str label text
 */
public SwingField(Field f, String str) {
    super();
    setText(str);
    setEditable(false);
    field = f;
    format();
}

public Dimension getPreferredSize() {
    return new Dimension((int)field.getBounds().width,
			 (int)field.getBounds().height);
}

/**
 * Returns the report field this label represents.
 *
 * @return a report field
 */
public Field getField() { return field; }

/**
 * Formats the label according to the field's formatting specifications.
 */
public void format() {
    Format format = field.getFormat() == null
	? Format.DEFAULT : field.getFormat();
    Style style = StyleContext.getDefaultStyleContext()
	.getStyle(StyleContext.DEFAULT_STYLE);

    // Save selection and select all text
    int selStart = getSelectionStart();
    int selEnd = getSelectionEnd();
    selectAll();

    StyleConstants.setBold(style, format.isBold());
    StyleConstants.setItalic(style, format.isItalic());
    StyleConstants.setUnderline(style, format.isUnderline());
    StyleConstants.setFontSize(style, (int)format.getSize());
    StyleConstants.setFontFamily(style, "Serif");

    // Align
    switch (format.getAlign()) {
    case Format.ALIGN_CENTER:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_CENTER);
	break;
    case Format.ALIGN_RIGHT:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_RIGHT);
	break;
    default:
	StyleConstants.setAlignment(style, StyleConstants.ALIGN_LEFT);
	break;
    }
    setParagraphAttributes(style, true);

    // Restore selection
    setCaretPosition(selStart);
    moveCaretPosition(selEnd);

    makeBorders();
}

/**
 * Makes borders using the field's border. <em>Unimplemented</em>.
 */
protected void makeBorders() {
//      if (field.getBorder() != null)
//  	field.getBorder().eachLine(this); // Calls drawLine for each line
}

//  public void drawLine(Line line) {
//  }


}
