package jimm.datavision.layout;
import jimm.datavision.*;

/**
 * An XML layout engine.
 *
 * @author Jim Menard, <a href="mailto:jimm@io.com">jimm@io.com</a>
 */
public class XMLLE extends LayoutEngine {

protected IndentWriter iout;

/**
 * Constructor.
 *
 * @param out an indent writer
 */
public XMLLE(IndentWriter out) {
    super();
    iout = out;
}

protected void doStart() {
    iout.println("<?xml version=\"1.0\"?>");
    iout.println("<report>");
    iout.indent();
}

protected void doEnd() {
    iout.outdent();
    iout.println("</report>");
    iout.flush();
}

protected void doStartPage() {
    iout.println("============== Page #{" + pageNumber() + "} ==============");
    iout.println("<newpage number=\"" + pageNumber() + "\" />");
}

protected void doOutputSection(Section s) {
    iout.println("<section type=\"" + currentSectionType + "\">");
    iout.indent();
    super.doOutputSection(s);
    iout.outdent();
    iout.println("</section>");
}

protected void doOutputField(Field field) {
    iout.print("<field id=\"" + field.getId() + "\" type=\""
	       + field.typeString() + "\"");
    if (field instanceof SpecialField)
	iout.print(" value=\"" + field.getValue() + "\"");
    else if (field instanceof ColumnField)
	iout.print(" column=\"" + ((ColumnField)field).getColumn().fullName()
		   + "\"");
    else if (field instanceof SubtotalField) {
	SubtotalField sf = (SubtotalField)field;
	if (sf.getGroup() != null)
	    iout.print(" group=\"" + sf.getGroup().getColumn().getId() + "\"");
    }

    iout.println("><![CDATA[" + field.toString() + "]]></field>");
}

protected void doOutputLine(Line l) {
    l.writeXML(iout);
}

}

