package jimm.datavision.test;
import jimm.datavision.Report;
import jimm.datavision.IndentWriter;
import jimm.datavision.LayoutEngine;
import jimm.datavision.layout.*;
import java.io.*;

public class ReportRunner {

/**
 * usage: ReportRunner (-c|-d|-h|-l|-s|-x) xml_file
 */
public static void main(String[] args) {
    if (args.length < 2 || args[0].length() < 2 || args[0].charAt(0) != '-')
	usage();

    try {
	LayoutEngine layout = null;

	switch (args[0].charAt(1)) {
	case 'c':
	    layout = new CharSepLE(new PrintWriter(new FileWriter("out.csv")),
				   ',');
	    break;
	case 'd':
	    layout =
		new DocBookLE(new PrintWriter(new FileWriter("out.sgml")));
	    break;
	case 'h':
	    layout = new HTMLLE(new PrintWriter(new FileWriter("out.html")));
	    break;
	case 'l':
	    layout = new LaTeXLE(new PrintWriter(new FileWriter("out.tex")));
	    break;
	case 's':
	    layout = new SwingLE();
	    break;
	case 'x':
	    layout = new XMLLE(new IndentWriter(new FileWriter("out.xml")));
	    break;
	default:
	    usage();
	    break;
	}

	if (layout != null) {
	    Report r = new Report(args[1]);
	    r.setLayoutEngine(layout);
	    r.run();
	}
    }
    catch (Exception e) {
	e.printStackTrace();
    }
}

protected static void usage() {
    System.err.println("usage: ReportRunner (-c|-d|-h|-l|-s|-x) xml_file");
    System.err.println("  first argument specifies layout engine:");
    System.err.println("  -c character-separated (uses comma;"
		       + " generates out.csv");
    System.err.println("  -d DocBook (generates out.sgml)");
    System.err.println("  -h HTML (generates out<n>.html files)");
    System.err.println("  -l LaTeX (generates out.tex)");
    System.err.println("  -s Swing (displays printable window)");
    System.err.println("  -x XML (generates out.xml)");
    System.exit(-1);
}

}
