package jimm.datavision.test;
import jimm.datavision.Formula;
import jimm.datavision.Utils;
import java.util.HashMap;
import java.util.Iterator;

public class TokenizerTest {

public static void main(String[] args) {
    utilTests();
    singleTokenTests();
    expressionTests();
}

public static void utilTests() {
    testEscape("abc&def", "abc&amp;def");
    testEscape("abc\"def'", "abc&quot;def&apos;");
    testEscape("<>", "&lt;&gt;");

    testUnescape("abc&amp;def", "abc&def");
    testUnescape("abc&quot;def&apos;", "abc\"def'");
    testUnescape("&lt;&gt;", "<>");
    testUnescape("abc&#20;def", "abc def");
}

protected static void testEscape(String question, String expectedAnswer) {
    String answer = Utils.escapeXML(question);
    if (!answer.equals(expectedAnswer))
	System.err.println("escape expected string \"" + expectedAnswer
			   + "\" but saw \"" + answer + "\"");
}

protected static void testUnescape(String question, String expectedAnswer) {
    String answer = Utils.unescapeXML(question);
    if (!answer.equals(expectedAnswer))
	System.err.println("unescape expected string \"" + expectedAnswer
			   + "\" but saw \"" + answer + "\"");
}

public static void singleTokenTests() {
    HashMap testVals = new HashMap();
    Double theAnswer = new Double(42);
    testVals.put("42", theAnswer);
    testVals.put(" 42", theAnswer);
    testVals.put("42 ", theAnswer);
    testVals.put(" 42 ", theAnswer);
    testVals.put(" \t 42 \t", theAnswer);
    testVals.put("66.6", new Double(66.6));
    testVals.put("\"foo\"", "\"foo\"");
    testVals.put("\"don't forget apostrophes\"",
		 "\"don't forget apostrophes\"");
    testVals.put("\"or \\\"quoted quotes\\\" either\"",
		 "\"or \\\"quoted quotes\\\" either\"");

    runTests(testVals);
}

public static void expressionTests() {
    HashMap testVals = new HashMap();
    Double theAnswer = new Double(42);
    testVals.put("39 3 +", theAnswer);
    testVals.put(" 39 3 + ", theAnswer);
    testVals.put(" 39\t3 +", theAnswer);
    testVals.put(" 39    3\t+", theAnswer);
    testVals.put("39 3 +", theAnswer);
    testVals.put("45 3 -", theAnswer);
    testVals.put("21 2 *", theAnswer);
    testVals.put("84.00 2.0 /", theAnswer);
    testVals.put(" \"foo\" \"bar\" + ", "\"foobar\"");
    testVals.put("\"foo\" 3 *", "foofoofoo");
    testVals.put("\"foo\" 3.5 *", "foofoofoo");

    runTests(testVals);
}

public static void runTests(HashMap testVals) {
    Formula f = new Formula(null, null, "Unnamed Formula");
    for (Iterator iter = testVals.keySet().iterator(); iter.hasNext(); ) {
	String formula = (String)iter.next();
	Object answer = testVals.get(formula);
	try {
	    Object calculated = f.rpnEval(formula);
	    if (!calculated.equals(answer)
		|| calculated.getClass() != answer.getClass())
	    {
		System.out.println("formula [" + formula + "] should eval to ["
				   + answer + "]; saw [" + calculated
				   + "] instead");
	    }
	}
	catch (Exception e) {
	    System.err.println("While evaluating formula [" + formula + "]:");
	    System.err.println(e.toString());
	}
    }
}

}
